# -*- coding: utf-8 -*-

# DO NOT EDIT THIS FILE!
# This file has been autogenerated by dephell <3
# https://github.com/dephell/dephell

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import os.path

readme = ''
here = os.path.abspath(os.path.dirname(__file__))
readme_path = os.path.join(here, 'README.md')
if os.path.exists(readme_path):
    with open(readme_path, 'rb') as stream:
        readme = stream.read().decode('utf8')

setup(
    long_description=readme,
    name='streamlit-inspect',
    version='0.1.0',
    description='Create interactive documentation using streamlit',
    python_requires='==3.*,>=3.6.0',
    project_urls={"repository": "https://github.com/cceyda/streamlit-inspect"},
    author='Ceyda Cinarel',
    author_email='15624271+cceyda@users.noreply.github.com',
    license='MIT',
    keywords='streamlit, streamlit-component',
    packages=['streamlit_inspect'],
    package_dir={"": "."},
    package_data={},
    install_requires=['streamlit==0.*,>=0.80.0',
                      'typing-inspect==0.*,>=0.6.0'],
    extras_require={"dev": ["pytest==4.*,>=4.6.0"]},
)
